<?php

namespace App\Service\Action\Classes;

use App\Entity\User;
use App\Service\Action\BaseActionService;
use App\Service\Dto\Classes\RegisterDto;
use App\Service\Dto\DtoServiceInterface;
use App\Service\Response\ResponseServiceInterface;
use Doctrine\Persistence\ManagerRegistry;
use ReflectionClass;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;

class Register extends BaseActionService
{
    /**
     * @param RegisterDto $registerDto
     * @param ResponseServiceInterface $profileResponse
     * @param UserPasswordHasherInterface $passwordHasher
     * @param ManagerRegistry $doctrine
     */
    public function __construct(
        private DtoServiceInterface         $registerDto,
        private ResponseServiceInterface    $profileResponse,
        private UserPasswordHasherInterface $passwordHasher,
        private ManagerRegistry             $doctrine,
    )
    {
        parent::__construct($profileResponse);
    }

    /**
     * Регистрация
     *
     * @return void
     */
    public function runAction(): void
    {
        $user = $this->createUser();
        if ($user !== null) {
            $userExists = $this->doctrine->getRepository(User::class)
                ->findOneByUniq($user->getEmail(), $user->getPhoneNumber());

            if ($userExists) {
                $this->profileResponse->getResponse()->addError('Пользователь уже существует');
            } else {
                try {
                    $user->setDeleted(false);
                    $user->setConfirm(false);
                    $hashedPassword = $this->passwordHasher->hashPassword(
                        $user,
                        $this->registerDto->getClass()->password ?: ''
                    );
                    $user->setPassword($hashedPassword);

                    $em = $this->doctrine->getManager();

                    $em->persist($user);
                    $em->flush();
                    $this->profileResponse->getResponse()->addMessage('Пользователь зарегистрирован');
                } catch (\Exception $exception) {
                    $this->profileResponse->getResponse()->addError('Ошибка регистрации пользователя');
                }

            }
        }
    }

    /**
     * Валидация
     *
     * @return bool
     */
    public function validate(): bool
    {
        return $this->registerDto->validate($this->profileResponse);
    }

    /**
     * Создание пользователя из Dto
     *
     * @return User|null
     */
    private function createUser(): ?User
    {
        $user = null;

        $data = $this->registerDto->toArray();

        if ($data) {
            $user = new User();

            $reflectionClass = new ReflectionClass($user);
            foreach ($reflectionClass->getProperties() as $property) {
                $type = $property->getType();
                if (isset($data[$property->getName()])) {
                    $sValue = $data[$property->getName()] ?: null;
                    if ($sValue !== null && $type !== null && ($type->getName() !== 'array')) {
                        $property->setValue($user, $sValue);
                    }
                }
            }
        } else {
            $this->profileResponse->getResponse()->addError('Ошибка получения данных');
        }

        return $user;
    }
}